/* ========================================================================
 * Copyright (c) 2009 The University of Washington
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================================================================
 */

package edu.washington.shibboleth.authn.provider;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.Cookie;

import java.io.IOException;
import java.util.List;
import java.util.Iterator;
import java.util.Enumeration;


import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.opensaml.util.URLBuilder;
import org.opensaml.saml2.core.AuthnContext;

import edu.internet2.middleware.shibboleth.idp.authn.provider.AbstractLoginHandler;
import edu.internet2.middleware.shibboleth.idp.authn.LoginHandler;
import edu.internet2.middleware.shibboleth.idp.authn.LoginContext;
import edu.internet2.middleware.shibboleth.idp.authn.ShibbolethSSOLoginContext;
import edu.internet2.middleware.shibboleth.idp.util.HttpServletHelper;

public class UWLoginHandler extends AbstractLoginHandler {

    private final Logger log = LoggerFactory.getLogger(UWLoginHandler.class);

    public static final String UW_LOGIN_SILVER_AUTHN_METHOD = "http://incommonfederation.org/assurance/silver";
    public static final String UW_LOGIN_SILVER_TEST_AUTHN_METHOD = "http://incommonfederation.org/assurance/silver-test";

    // plainAuthn servlet URL
    private String plainAuthnURL;

    // forceAuthn servlet URL
    private String forceAuthnURL;

    // passive servlet URL
    private String passiveAuthnURL;

    // token servlet URL
    private String tokenAuthnURL;

    // silver servlet URL
    private String silverAuthnURL;

    // sso session cookie name
    private String ssoCookieName;

    /**
     * Constructor.
     *
     * @param URLs to the authentication servlets
     */
   public UWLoginHandler(String plain, String force, String passive, String token, String silver, String cookie) {
        super();
           log.debug("UWLoginHandler constructor");
        setSupportsPassive(false);
        setSupportsForceAuthentication(true);
        plainAuthnURL = plain;
        forceAuthnURL = force;
        passiveAuthnURL = passive;
        tokenAuthnURL = token;
        silverAuthnURL = silver;
        ssoCookieName = cookie;
    }

    public void login(final HttpServletRequest httpRequest, final HttpServletResponse httpResponse) {

        log.debug("UWLogin login");

        // LoginContext loginContext = (LoginContext) httpRequest.getAttribute(LoginContext.LOGIN_CONTEXT_KEY);
        LoginContext loginContext = HttpServletHelper.getLoginContext(httpRequest);
        if (loginContext==null) {
           log.error("No login context!? Unable to redirect to authentication servlet.");
           return;
        }
        String attemptedMethod = loginContext.getAttemptedAuthnMethod();
        log.debug(".. attempting: " + attemptedMethod);

        String authnURL = plainAuthnURL;
        String authnMethod = AuthnContext.PPT_AUTHN_CTX;

        if (loginContext.isForceAuthRequired()) {
           authnURL = forceAuthnURL;
           log.debug("uwlogin: forceAuthn, sso cookie=" + ssoCookieName);

           // need to clear weblogin's session cookies
           Cookie[] cookies = httpRequest.getCookies();
           if (cookies != null && ssoCookieName != null) {
                for (Cookie cookie : cookies) {
                    if (cookie.getName().matches(ssoCookieName)) {
                        log.debug(" uwlogin:  clearing session cookie:" + cookie.getName());
                        Cookie voidCookie = new Cookie(cookie.getName(), "");
                        voidCookie.setMaxAge(0);
                        voidCookie.setPath("/");    // cookie path should be config?
                        voidCookie.setSecure(true);
                        httpResponse.addCookie(voidCookie);
                    }
                }
            }
        }

        if (loginContext.isPassiveAuthRequired()) {
           authnURL = passiveAuthnURL;
           log.debug("uwlogin:  passive requested");
        }

        if (attemptedMethod.equals(UW_LOGIN_SILVER_AUTHN_METHOD)) {
           log.debug("uwlogin:  is silver");  
           authnURL = silverAuthnURL;
           authnMethod = UW_LOGIN_SILVER_AUTHN_METHOD;

        } else if (attemptedMethod.equals(UW_LOGIN_SILVER_TEST_AUTHN_METHOD)) {
           log.debug("uwlogin:  is silver test");  
           authnURL = silverAuthnURL;
           authnMethod = UW_LOGIN_SILVER_TEST_AUTHN_METHOD;

        } else if (attemptedMethod.equals(AuthnContext.TIME_SYNC_TOKEN_AUTHN_CTX)) {
           log.debug("uwlogin:  is securid");  
           authnURL = tokenAuthnURL;
           authnMethod = AuthnContext.TIME_SYNC_TOKEN_AUTHN_CTX;
        }

        log.debug("uwlogin:  url: " + authnURL);
        httpRequest.setAttribute(LoginHandler.AUTHENTICATION_METHOD_KEY, authnMethod);

        // forward control to the servlet
        try {
            StringBuilder pathBuilder = new StringBuilder();
            pathBuilder.append(httpRequest.getContextPath());
            if(!authnURL.startsWith("/")){
                pathBuilder.append("/");
            }

            pathBuilder.append(authnURL);

            URLBuilder urlBuilder = new URLBuilder();
            urlBuilder.setScheme(httpRequest.getScheme());
            urlBuilder.setHost(httpRequest.getServerName());
            urlBuilder.setPort(httpRequest.getServerPort());
            urlBuilder.setPath(pathBuilder.toString());

            log.debug("Redirecting to {}", urlBuilder.buildURL());
            httpResponse.sendRedirect(urlBuilder.buildURL());
            return;

        } catch (IOException ex) {
            log.error("Unable to redirect to authentication servlet.", ex);
        }
    }
}
