/* ========================================================================
 * Copyright (c) 2009 The University of Washington
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================================================================
 */

package edu.washington.shibboleth.authn.provider;

import java.io.IOException;
import java.util.Map;
import java.util.Enumeration;

import javax.servlet.ServletContext;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.opensaml.saml2.metadata.provider.MetadataProvider;
import org.opensaml.saml2.metadata.provider.MetadataProviderException;
import org.opensaml.util.storage.StorageService;


import edu.internet2.middleware.shibboleth.common.attribute.BaseAttribute;
import edu.internet2.middleware.shibboleth.common.attribute.provider.SAML2AttributeAuthority;
import edu.internet2.middleware.shibboleth.common.attribute.AttributeRequestException;
import edu.internet2.middleware.shibboleth.common.attribute.resolver.AttributeResolver;

import edu.internet2.middleware.shibboleth.common.profile.provider.BaseSAMLProfileRequestContext;
import edu.internet2.middleware.shibboleth.common.relyingparty.RelyingPartyConfiguration;
import edu.internet2.middleware.shibboleth.common.relyingparty.RelyingPartyConfigurationManager;
import edu.internet2.middleware.shibboleth.common.relyingparty.provider.SAMLMDRelyingPartyConfigurationManager;

import edu.internet2.middleware.shibboleth.idp.authn.AuthenticationEngine;
import edu.internet2.middleware.shibboleth.idp.authn.LoginHandler;
import edu.internet2.middleware.shibboleth.idp.authn.LoginContext;
import edu.internet2.middleware.shibboleth.idp.authn.LoginContextEntry;
import edu.internet2.middleware.shibboleth.idp.authn.PassiveAuthenticationException;
import edu.internet2.middleware.shibboleth.idp.util.HttpServletHelper;




/**
 * Extracts the REMOTE_USER and places it in a request attribute to be used by the authentication engine.
 * Allows passive authn
 */
public class UWLoginServlet extends HttpServlet {

    /** Serial version UID. */
    private static final long serialVersionUID = 1745454088856633626L;

    /** Class logger. */
    private final Logger log = LoggerFactory.getLogger(UWLoginServlet.class);

    /* Servlet Context */
    private static ServletContext context;
    SAML2AttributeAuthority attributeAuthority;
    AttributeResolver attributeResolver;
    SAMLMDRelyingPartyConfigurationManager relyingPartyConfigurationManager;
    private static StorageService<String, LoginContextEntry> storageService;

    /** {@inheritDoc} */
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        log.debug("UWLoginServlet init");
        context = config.getServletContext();
        attributeAuthority = HttpServletHelper.getSAML2AttributeAuthority(context);
        if (attributeAuthority!=null) log.debug("UWLoginServlet have authority");
        attributeResolver = HttpServletHelper.getAttributeResolver(context);
        if (attributeResolver!=null) log.debug("UWLoginServlet have Resolver");
        relyingPartyConfigurationManager = (SAMLMDRelyingPartyConfigurationManager) HttpServletHelper.getRelyingPartyConfirmationManager(context);
        if (relyingPartyConfigurationManager!=null) log.debug("UWLoginServlet have configurationManager");
        storageService = (StorageService<String, LoginContextEntry>) HttpServletHelper.getStorageService(context);
        if (storageService!=null) log.debug("UWLoginServlet have storageService");
        log.debug("UWLoginServlet init done");
    }

    /** {@inheritDoc} */
    protected void service(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws ServletException,
            IOException {
        log.debug("UWLoginServlet service");

        String principalName = httpRequest.getRemoteUser();
        LoginContext loginContext = HttpServletHelper.getLoginContext(storageService, context, httpRequest);
        if (loginContext!=null) log.debug("UWLoginServlet have login context");
        else log.error(".. no login context");

        BaseSAMLProfileRequestContext requestContext = createRequestContext(loginContext, principalName);
        
        log.debug("UWLoginServlet get attrs");
        try {
            log.debug("Resolving attributes for principal '{}' for SAML request from relying party '{}'",
                    principalName, loginContext.getRelyingPartyId());
            Map<String, BaseAttribute> principalAttributes = attributeAuthority.getAttributes(requestContext);
            requestContext.setAttributes(principalAttributes);
        } catch (AttributeRequestException e) {
            log.warn("Error resolving attributes for principal '{}'.  No name identifier or attribute statement will be included in response",
                    requestContext.getPrincipalName());
        }

        /// what to do??

        if (principalName!=null) {
           log.debug("Remote user identified as {} returning control back to authentication engine", principalName);
           httpRequest.setAttribute(LoginHandler.PRINCIPAL_NAME_KEY, httpRequest.getRemoteUser());
           AuthenticationEngine.returnToAuthenticationEngine(httpRequest, httpResponse);
        } else {
           // loginContext.setAuthenticationFailure(new PassiveAuthenticationException("no passive to be found"));
           httpRequest.setAttribute(LoginHandler.AUTHENTICATION_ERROR_KEY, "No user session available for passive login");
           log.debug("Passibve1 remote user not available, returning control back to authentication engine");
           AuthenticationEngine.returnToAuthenticationEngine(httpRequest, httpResponse);

           // AuthenticationEngine.returnToProfileHandler(loginContext, httpRequest, httpResponse);
        }
    }


    private BaseSAMLProfileRequestContext createRequestContext(LoginContext loginContext, String principalName) {
        log.debug("UWLoginServlet createRequestContext");
        BaseSAMLProfileRequestContext context = new BaseSAMLProfileRequestContext();
        log.debug("UWLoginServlet create1");
        String rpId = loginContext.getRelyingPartyId();
        log.debug("UWLoginServlet rpid = " + rpId);
        RelyingPartyConfiguration relyingPartyConfiguration = relyingPartyConfigurationManager.getRelyingPartyConfiguration(rpId);
        if (relyingPartyConfiguration!=null) log.debug("UWLoginServlet have relyingPartyConfiguration");
        MetadataProvider metadataProvider = relyingPartyConfigurationManager.getMetadataProvider();
        if (metadataProvider!=null) log.debug("UWLoginServlet have metadataProvider");
        String idpId = relyingPartyConfiguration.getProviderId();
        context.setRelyingPartyConfiguration(relyingPartyConfiguration);
        context.setInboundMessageIssuer(rpId);
        context.setInboundMessageIssuer(idpId);
        context.setPrincipalName(principalName);
        context.setLocalEntityId(idpId);
        context.setPeerEntityId(rpId);
        context.setMetadataProvider(metadataProvider);

        try {
           context.setLocalEntityMetadata(metadataProvider.getEntityDescriptor(idpId));
           context.setPeerEntityMetadata(metadataProvider.getEntityDescriptor(rpId));
        } catch (MetadataProviderException e) {
           log.error("no metadata" +  e);
           return null;
        }
        
        // this enough stuff?
        log.debug("UWLoginServlet createRequestContext done");
        return context;
    }
}
